     //////////////////////////////////////////////////////////////////////
     // Copyright (c) 1999 David Wallace Croft (croft@alumni.caltech.edu)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.app.hitcounter;

     import java.io.*;
     import java.util.*;

     /*********************************************************************
     * Hit counter data for an individual web page.
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1999-09-12
     *********************************************************************/

     public class  HitCounterPageData implements Serializable, Comparable
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private static final long  serialVersionUID = 1L;

     private String  name;

     private Date    firstHitDate;

     private long    hits;

     //////////////////////////////////////////////////////////////////////
     // Static methods
     //////////////////////////////////////////////////////////////////////

     public static String  normalizeURLName ( String  name )
     //////////////////////////////////////////////////////////////////////
     // This could be made more efficient so that it does not create
     // and discard so many temporary Strings.
     //////////////////////////////////////////////////////////////////////
     {
       if ( name == null ) name = "null";

       int  cutoff = name.indexOf ( '?' );

       if ( cutoff > -1 ) name = name.substring ( 0, cutoff );

       cutoff = name.indexOf ( '#' );

       if ( cutoff > -1 ) name = name.substring ( 0, cutoff );

       return name;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  HitCounterPageData ( String  name )
     //////////////////////////////////////////////////////////////////////
     {
       if ( name == null )
       {
         throw new IllegalArgumentException ( "null name" );
       }

       this.name = normalizeURLName ( name );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public String  getName ( )
     //////////////////////////////////////////////////////////////////////
     {
       return name;
     }

     public Date  getFirstHitDate ( )
     //////////////////////////////////////////////////////////////////////
     {
       return firstHitDate;
     }

     public synchronized long  getHits ( )
     //////////////////////////////////////////////////////////////////////
     {
       return hits;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public synchronized long  incrementHits ( )
     //////////////////////////////////////////////////////////////////////
     {
       if ( firstHitDate == null )
       {
         firstHitDate = new Date ( );
       }

       return ++hits;
     }

     public double  getHitsPerDay ( long  now )
     //////////////////////////////////////////////////////////////////////
     {
       if ( firstHitDate == null )
       {
         return 0.0;
       }
       else
       {
         double  days = now - firstHitDate.getTime ( );

         days /= 1000 * 60 * 60 * 24;

         return getHits ( ) / days;
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Compares by hits per day.
     *********************************************************************/
     public int  compareTo ( Object  other )
     //////////////////////////////////////////////////////////////////////
     {
       if ( other == null )
       {
         return 1;
       }

       HitCounterPageData  that = ( HitCounterPageData ) other;

       long  now = new Date ( ).getTime ( );

       double  thisHitsPerDay = this.getHitsPerDay ( now );

       double  thatHitsPerDay = that.getHitsPerDay ( now );

       if ( thisHitsPerDay > thatHitsPerDay )
       {
         return 1;
       }
       else if ( thisHitsPerDay < thatHitsPerDay )
       {
         return -1;
       }
       else
       {
         return 0;
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
