     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.lang;

     /*********************************************************************
     * <P>
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   1998-02-09
     *********************************************************************/

     public final class  ParseLib {
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     private  ParseLib ( ) { }

     /*********************************************************************
     * Returns backup if s is null or a NumberFormatException occurs.
     *********************************************************************/
     public static int  parseInt ( String  s, int  backup ) {
     //////////////////////////////////////////////////////////////////////
       if ( s == null ) return backup;
       try { return Integer.parseInt ( s ); }
       catch ( NumberFormatException  e ) { return backup; }
     }

     /*********************************************************************
     * Returns <I>backup</I> if s is null, less than min, greater than max,
     * or a NumberFormatException occurs.
     *********************************************************************/
     public static int  parseInt (
       String  s, int  backup, int  min, int  max )
     //////////////////////////////////////////////////////////////////////
     {
       int  parsed = parseInt ( s, backup );
       if ( parsed < min ) return backup;
       if ( parsed > max ) return backup;
       return parsed;
     }

     /*********************************************************************
     * Returns true  if s is "1" or starts with "t", "T", "y", or "Y".
     * Returns true  if s is "on", case-insensitive.
     * Returns false if s is "0" or starts with "f", "F", "n", or "N".
     * Returns false if s is "off", case-insenstive.
     * Returns backup if s is null or other.
     *********************************************************************/
     public static boolean  parseBoolean ( String  s, boolean  backup ) {
     //////////////////////////////////////////////////////////////////////
       if ( s == null ) return backup;
       if ( s.equals ( "0" ) ) return false;
       if ( s.equals ( "1" ) ) return true;
       s = s.toLowerCase ( );
       if ( s.startsWith ( "f" ) ) return false;
       if ( s.startsWith ( "t" ) ) return true;
       if ( s.startsWith ( "n" ) ) return false;
       if ( s.startsWith ( "y" ) ) return true;
       if ( s.equals ( "on" ) ) return true;
       if ( s.equals ( "off" ) ) return true;
       return backup;
     }

     /*********************************************************************
     * Returns backup if s is null or a NumberFormatException occurs.
     *********************************************************************/
     public static double  parseDouble ( String  s, double  backup ) {
     //////////////////////////////////////////////////////////////////////
       if ( s == null ) return backup;
       try { return Double.valueOf ( s ).doubleValue ( ); }
       catch ( NumberFormatException  e ) { return backup; }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
