     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.util.param;

     /*********************************************************************
     *
     * Stores zero or more String values as keyed by unique String names.
     * <P>
     * This class is similar to java.util.Properties except that the
     * number of values associated with each parameter name does not have
     * to be exactly one.
     * <P>
     * The static methods in ParametersLib provides optional default
     * implementations for classes that use this interface.
     *
     * @see
     *   ParametersLib
     * @see
     *   java.util.Properties
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1998-11-02
     *********************************************************************/

     public interface  Parameters
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     /*********************************************************************
     * Adds the <I>parameterName</I> with an empty array of parameter
     * values.
     *
     * @return
     *   Returns false if the name already exists.
     * @exception  IllegalArgumentException
     *   If <I>parameterName</I> is null.
     *********************************************************************/
     public boolean  addName ( String  parameterName );

     /*********************************************************************
     * Returns the first value of the specified parameter name.
     * Returns null if the parameter name or value does not exist.
     *
     * @exception IllegalArgumentException
     *   If <I>parameterName</I> is null.
     *********************************************************************/
     public String  getFirstValue ( String  parameterName );

     /*********************************************************************
     * Returns the first value of the specified parameter name.
     * Returns <I>defaultValue</I> if the parameter name or value does not
     * exist.
     *
     * @exception IllegalArgumentException
     *   If <I>parameterName</I> is null.
     *********************************************************************/
     public String  getFirstValue (
       String  parameterName, String  defaultValue );

     /*********************************************************************
     * Returns a String array of the parameter names.
     * Returns an empty array if there are no parameter names.
     *********************************************************************/
     public String [ ]  getNames ( );

     /*********************************************************************
     * Returns a String array of values of the specified parameter name.
     * Returns null if the named parameter does not exist.
     * Returns an empty array if no values exist for the name.
     *
     * @exception IllegalArgumentException
     *   If <I>parameterName</I> is null.
     *********************************************************************/
     public String [ ]  getValues ( String  parameterName );

     /*********************************************************************
     * Removes the <I>parameterName</I> from this container.
     *
     * @return
     *   The parameter values or null if the <I>parameterName</I> could not
     *   be removed.
     * @exception IllegalArgumentException
     *   If <I>parameterName</I> is null.
     *********************************************************************/
     public String [ ]  removeName ( String  parameterName );

     /*********************************************************************
     * Sets the <I>parameterValues</I> for the given <I>parameterName</I>.
     *
     * @param  parameterValues
     *   May be an empty array of String.
     * @return
     *   The previous set of parameter values.
     * @exception IllegalArgumentException
     *   If <I>parameterName</I> or <I>parameterValues</I> is null.
     *********************************************************************/
     public String [ ]  setValues (
       String  parameterName, String [ ]  parameterValues );

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
