     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.app.chat.ChatMessage",
     // created 1999-12-24.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 2000 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.chat;

     import java.io.*;
     import java.net.SocketException;

     import com.orbs.open1.mpl.io.StreamLib;

     import com.orbs.open1.mit.util.param.*;

     /*********************************************************************
     *
     * <P>
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   2000-04-27
     *********************************************************************/

     public final class  ChatMessage
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     // Send a line of chat text.

     public static final String  CODE_REQUEST_CHAT    = "QC";

     // Request an EntityID.

     public static final String  CODE_REQUEST_ENTITY  = "QE";

     // Request change of ChatThing image to given URL.

     public static final String  CODE_REQUEST_IMAGE   = "QI";

     // Load model mirror (info about other ChatThings).

     public static final String  CODE_REQUEST_LOAD    = "QL";

     // Download any waiting responses.

     public static final String  CODE_REQUEST_MESSAGE = "QM";

     // Move ChatThing to a new location.

     public static final String  CODE_REQUEST_MOVE    = "QN";

     // Create a ChatThing.

     public static final String  CODE_REQUEST_THING   = "QT";

     // Reset the view dimensions.

     public static final String  CODE_REQUEST_VIEW    = "QV";

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     // A newly assigned ChatEntityID.

     public static final String  CODE_RESPONSE_ENTITY = "RE";

     //

     public static final String  CODE_RESPONSE_LOAD   = "RL";

     // Move.

     public static final String  CODE_RESPONSE_MOVE   = "RM";

     // Error.

     public static final String  CODE_RESPONSE_ERROR  = "RR";

     // Thing creation.

     public static final String  CODE_RESPONSE_THING  = "RT";

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static final String  CODE_EVENT_CHAT   = "EC";

     // Indicates that a ChatThing has changed its imageURLName.

     public static final String  CODE_EVENT_IMAGE  = "EI";

     public static final String  CODE_EVENT_MOVE   = "EM";

     // Indicates the removal of a ChatThing from the world.

     public static final String  CODE_EVENT_REMOVE = "ER";

     // Indicates the creation of a new ChatThing in the world.

     public static final String  CODE_EVENT_THING  = "ET";

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     private static final String  PARAM_MESSAGE_CODE   = "C";

     private static final String  PARAM_CHAT_ENTITY_ID = "E";

     private static final String  PARAM_VALUES         = "V";


     private String        messageCode;

     private ChatEntityID  chatEntityID;

     private String [ ]    values;

//     private final long        messageID;

//     private final String      messageDigest;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static ChatMessage  parse (
       InputStream  inputStream,
       int          contentLength )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       if ( contentLength < 0 )
       {
         return null;

// need to modify to handle -1, unknown length
       }

       byte [ ]  bytes = new byte [ contentLength ];

       int  length = inputStream.read ( bytes );

       String  msgStr =  new String ( bytes, 0, length );

       try
       {
         return decode ( msgStr );
       }
       catch ( IllegalArgumentException  ex )
       {
//System.err.println ( "ChatMessage.parse() error:  " + msgStr );

return null;
       }
     }

     public static ChatMessage  decode ( String  s )
     //////////////////////////////////////////////////////////////////////
     {
       if ( s == null )
       {
         throw new IllegalArgumentException ( "null s" );
       }

System.err.println ( "ChatMessage.decode():  " + s );

       return new ChatMessage (
         ParametersLib.parseHttpGetParameters ( s ) );
     }

     public static ChatMessage  decode ( Parameters  parameters )
     //////////////////////////////////////////////////////////////////////
     {
/*
String [ ]  names = parameters.getNames ( );

System.err.println ( "ChatMessage.decode() names.length:  " + names.length );

for ( int  i = 0; i < names.length; i++ )
{
  System.err.println (
    "ChatMessage.decode() names[" + i + "]:  " + names [ i ] );
}
*/

       String  messageCode
         = parameters.getFirstValue ( PARAM_MESSAGE_CODE );

       ChatEntityID  chatEntityID = null;

       String  chatEntityIDStr
         = parameters.getFirstValue ( PARAM_CHAT_ENTITY_ID );

       if ( chatEntityIDStr != null )
       {
         chatEntityID = new ChatEntityID ( chatEntityIDStr );
       }

       String [ ]  values = parameters.getValues ( PARAM_VALUES );

       return new ChatMessage ( messageCode, chatEntityID, values );
     }

     public static String  encode ( ChatMessage  chatMessage )
     //////////////////////////////////////////////////////////////////////
     {
       return ParametersLib.urlEncode ( chatMessage.toParameters ( ) );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  ChatMessage (
       String        messageCode,
       ChatEntityID  chatEntityID,
       String [ ]    values )
     //////////////////////////////////////////////////////////////////////
     {
       if ( messageCode == null )
       {
         throw new IllegalArgumentException ( "null messageCode" );
       }

       this.messageCode  = messageCode;

       this.chatEntityID = chatEntityID;

       this.values       = values;
     }

     public  ChatMessage ( Parameters  parameters )
     //////////////////////////////////////////////////////////////////////
     {
       this (
         parameters.getFirstValue ( PARAM_MESSAGE_CODE ),
         parameters.getFirstValue ( PARAM_CHAT_ENTITY_ID ) == null
           ? null
           : new ChatEntityID (
             parameters.getFirstValue ( PARAM_CHAT_ENTITY_ID ) ),
         parameters.getValues ( PARAM_VALUES ) );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public String  getMessageCode ( )
     //////////////////////////////////////////////////////////////////////
     {
       return messageCode;
     }

     public ChatEntityID  getChatEntityID ( )
     //////////////////////////////////////////////////////////////////////
     {
       return chatEntityID;
     }

     public String [ ]  getValues ( )
     //////////////////////////////////////////////////////////////////////
     {
       return values;
     }

     public Parameters  toParameters ( )
     //////////////////////////////////////////////////////////////////////
     {
       Parameters  parameters = new ParametersHashtable ( );

       parameters.setValues (
         PARAM_MESSAGE_CODE, new String [ ] { messageCode } );

       if ( chatEntityID != null )
       {
         parameters.setValues ( PARAM_CHAT_ENTITY_ID,
           new String [ ] { chatEntityID.toString ( ) } );
       }

       if ( values != null )
       {
         parameters.setValues ( PARAM_VALUES, values );
       }

       return parameters;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
