     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.app.chat.client.ViewCanvas",
     // created 1998-05-30.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.chat.client;

     import java.applet.*;
     import java.awt.*;
     import java.awt.image.*;
     import java.io.*;
     import java.net.*;
     import java.util.*;

     import com.orbs.open1.mpl.animation.*;

     import com.orbs.open1.mpl.app.chat.*;

     /*********************************************************************
     * @version
     *   1999-12-25
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     *********************************************************************/

     public final class  ViewCanvas extends SpriteCanvas
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private static final String  BACKGROUND_IMAGE_PATH_FILENAME
       = "chat_images/ERNIE.GIF";

     private static final Color  BACKGROUND_COLOR = Color.black;

     private static final int    SQUARE           = 32;

     private ChatClient  chatClient;

     private ChatDisplay   chatDisplay;

     private Applet      applet;

     private Dimension   size = new Dimension ( );

     private URL         codeBaseURL;

     private Image       image_to_check;

     private boolean     image_ok;

     private Point       translation;

     private Image       backgroundImage;

     private Graphics    backgroundGraphics;

     private Sprite      sprite;

     private boolean     shutdown = false;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  ViewCanvas (
       ChatClient  chatClient,
       ChatDisplay   chatDisplay,
       Applet      applet )
     //////////////////////////////////////////////////////////////////////
     {
       super ( BACKGROUND_COLOR );

       this.chatClient = chatClient;

       this.chatDisplay  = chatDisplay;

       this.applet     = applet;

       if ( applet != null )
       {
         codeBaseURL = applet.getCodeBase ( );
       }

       translation = new Point ( 0, 0 );
     }

     public void  load_graphics ( )
     //////////////////////////////////////////////////////////////////////
     {
       backgroundImage = imageCache ( BACKGROUND_IMAGE_PATH_FILENAME );
     }

     public void  paint ( Graphics  g )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         if ( backgroundGraphics == null )
         {
           handleReshape ( );
         }

         super.paint ( g );

       }
       catch ( Exception  e )
       {
         e.printStackTrace ( );
       }
     }

     /*********************************************************************
     * Reshapes the Canvas.
     *********************************************************************/
     public synchronized void  reshape (
       int  x, int  y, int  width, int  height )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         super.reshape ( x, y, width, height );

         if ( !size ( ).equals ( this.size ) )
         {
           handleReshape ( );
         }
       }
       catch ( Exception  e )
       {
         e.printStackTrace ( );
       }
     }

     public boolean  imageUpdate (
       Image  image, int  flags, int  x, int  y, int  w, int  h )
     //////////////////////////////////////////////////////////////////////
     {
       if ( image_to_check == image )
       {
         image_ok = ( ( flags & ImageObserver.ALLBITS ) != 0 );
       }

       return super.imageUpdate ( image, flags, x, y, w, h );
     }

     private void  resized_drawImage (
       Graphics  g, Image  image,
       int  x, int  y, int  width, int  height )
     //////////////////////////////////////////////////////////////////////
     {
       image_ok = false;

       image_to_check = image;

       if ( !prepareImage ( image, width, height, this ) )
       {
         for ( int  i = 0; i < 4; i++ )
         {
           if ( image_ok )
           {
             break;
           }

           try
           {
             Thread.sleep ( i * 100 );
           }
           catch ( InterruptedException  e )
           {
           }
         }
       }

       g.drawImage ( image, x, y, width, height, this );
     }

     private Image  imageCache ( String  image_pathfile )
     //////////////////////////////////////////////////////////////////////
     {
       if ( image_pathfile == null ) return null;

       Image  image = null;

       if ( applet != null )
       {
         image = applet.getImage ( codeBaseURL, image_pathfile );
       }
       else
       {
         try
         {
           image = loadImage ( image_pathfile );
         }
         catch ( IOException  ex ) { return null; }
       }

       MediaTracker  mediaTracker = new MediaTracker ( this );

       mediaTracker.addImage ( image, 0 );

       try
       {
         mediaTracker.waitForAll ( 30000 );
       }
       catch ( InterruptedException  e )
       {
       }

       if ( !mediaTracker.checkAll ( ) || mediaTracker.isErrorAny ( ) )
       {
         System.err.println ( "ERROR loading image \""
           + image_pathfile + "\"" );

         return null;
       }

       return image;
     }

     private Image  loadImage ( String  image_pathfile )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       ClassLoader  classLoader = this.getClass ( ).getClassLoader ( );

if ( classLoader == null ) System.out.println ( "null ClassLoader" );

       InputStream  inputStream
         = classLoader.getResourceAsStream ( image_pathfile );

       BufferedInputStream  in = new BufferedInputStream ( inputStream );

       ByteArrayOutputStream  out = new ByteArrayOutputStream ( );

       int  i;

       while ( ( i = in.read ( ) ) > -1 )
       {
         out.write ( i );
       }

       byte [ ]  imageData = out.toByteArray ( );

       out.close ( );

       in.close ( );

       return Toolkit.getDefaultToolkit ( ).createImage ( imageData );
     }

     /*********************************************************************
     * index is the position in the sprite_Vector.
     * x and y are in universal coordinates.
     *********************************************************************/
     public synchronized void  avatarRelocate (
       ChatThingID  chatThingID,
       int          x,
       int          y )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         long  id = chatThingID.getL ( ).longValue ( );

// this could crash
         slide ( id, x - translation.x, y - translation.y );
//System.gc ( );
       }
       catch ( Exception  e )
       {
         e.printStackTrace ( );
       }
     }

     protected synchronized void  avatar_locate (
       long    avatar_id,
       String  avatar_image_pathfilename,
       int     x,
       int     y )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         Sprite  sprite = get_sprite ( avatar_id );

         Image  image = imageCache ( avatar_image_pathfilename );

         if ( sprite == null )
         {
           chatDisplay.display (
             "Creating sprite for Avatar #" + avatar_id );

// Need to do some coordinate transforms or something

           sprite = new Sprite ( this, image, ( int ) x, ( int ) y, 0,
             SQUARE, SQUARE, avatar_id );

           add ( sprite );

           chatDisplay.display ( "Sprite added for Avatar #" + avatar_id );
         }
         else
         {
           chatDisplay.display ( "Setting image for Avatar #" + avatar_id );

           sprite.setImage ( image );

           slip ( sprite, ( int ) x, ( int ) y );
         }
       } catch ( Exception  e )
       {
         e.printStackTrace ( );
       }
     }

     public synchronized boolean  mouseUp ( Event  event, int  x, int  y )
     //////////////////////////////////////////////////////////////////////
     {
// this needs work on translation

       chatClient.avatarRelocate ( x + translation.x, y + translation.y );

       return true;
     }

     protected synchronized void  shutdown ( )
     //////////////////////////////////////////////////////////////////////
     {
       if ( shutdown )
       {
         return;
       }

       shutdown = true;

       try
       {
         backgroundGraphics.dispose ( );
       }
       catch ( Throwable  t )
       {
         t.printStackTrace ( );
       }

       backgroundGraphics = null;
     }

     protected synchronized void  select_avatar_image (
       long  avatar_id, String  avatar_image_pathfilename )
     //////////////////////////////////////////////////////////////////////
     {
       chatDisplay.display ( "Selecting image for Avatar #" + avatar_id );

       Sprite  sprite = ( Sprite ) get_sprite ( avatar_id );

       if ( sprite == null )
       {
         avatar_locate ( avatar_id, avatar_image_pathfilename, 0, 0 );
       }
       else
       {
         setImage ( avatar_id, imageCache ( avatar_image_pathfilename ) );
       }
     }

     protected synchronized Rectangle  get_view ( )
     //////////////////////////////////////////////////////////////////////
     {
// Need to translate coordinates here
       Rectangle  r = bounds ( );

       r.x = 0;

       r.y = 0;

       return r;
     }

     protected synchronized void  quit ( long  avatar_id )
     //////////////////////////////////////////////////////////////////////
     {
       Sprite  sprite = get_sprite ( avatar_id );

       if ( sprite != null )
       {
         remove ( sprite );
       }
     }

     //////////////////////////////////////////////////////////////////////
     // Package protection methods
     //////////////////////////////////////////////////////////////////////

     synchronized void  handleReshape ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         this.size = size ( );

if ( ( size.width  < 1 )
  || ( size.height < 1 ) )
{
  System.out.println ( "size or height < 1" );

  return;
}

         if ( backgroundGraphics != null )
         {
           backgroundGraphics.dispose ( );
         }

         Image  backgroundImage = createImage ( size.width, size.height );

         if ( backgroundImage == null )
         {
           return;
         }

         backgroundGraphics = backgroundImage.getGraphics ( );

//       System.gc ( );

         backgroundGraphics.setColor ( BACKGROUND_COLOR );

System.out.println ( "ViewCanvas.handleReshape():  " + size.width + ", " + size.height );

         backgroundGraphics.fillRect ( 0, 0, size.width, size.height );

         if ( backgroundImage != null )
         {
           resized_drawImage (
             backgroundGraphics, backgroundImage, 0, 0, size.width, size.height );

           setBackgroundImage ( backgroundImage );
         }

         chatDisplay.display ( "Setting view..." );

         chatClient.setView ( );
       }
       catch ( Exception  e )
       {
         e.printStackTrace ( );
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
