     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "Mini".
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.compiler.mini.node;

     import java.util.*;

     /*********************************************************************
     * Parse tree node for the Mini programming language.
     *
     * @see
     *   MiniNode
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1999-04-27
     *********************************************************************/

     public class  ProcedureDeclarationMiniNode extends AbstractMiniNode
       implements DeclarationMiniNode
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     protected NameMiniNode               nameMiniNode;
     protected ParameterSequenceMiniNode  parameterSequenceMiniNode;
     protected BlockMiniNode              blockMiniNode;

     protected Boolean                    containsNested;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  ProcedureDeclarationMiniNode (
       NameMiniNode               nameMiniNode,
       ParameterSequenceMiniNode  parameterSequenceMiniNode,
       BlockMiniNode              blockMiniNode )
     //////////////////////////////////////////////////////////////////////
     {
       this.nameMiniNode              = nameMiniNode;
       this.parameterSequenceMiniNode = parameterSequenceMiniNode;
       this.blockMiniNode             = blockMiniNode;
     }

     //////////////////////////////////////////////////////////////////////
     // Access methods
     //////////////////////////////////////////////////////////////////////

     public NameMiniNode  getNameMiniNode ( ) { return nameMiniNode; }

     public ParameterSequenceMiniNode  getParameterSequenceMiniNode ( )
     //////////////////////////////////////////////////////////////////////
     {
       return parameterSequenceMiniNode;
     }

     public BlockMiniNode  getBlockMiniNode ( ) { return blockMiniNode; }

     public Boolean  containsNested ( ) { return containsNested; }

     public NameSequenceMiniNode  getNameSequenceMiniNode ( )
     //////////////////////////////////////////////////////////////////////
     {
       return new NameSequenceMiniNode ( nameMiniNode );
     }

     //////////////////////////////////////////////////////////////////////
     // MiniNode interface methods
     //////////////////////////////////////////////////////////////////////

     public void  generate ( MiniNodeCodeVisitor  miniNodeCodeVisitor )
     //////////////////////////////////////////////////////////////////////
     {
       miniNodeCodeVisitor.generateProcedureDeclaration ( this );
     }
     
     public void  checkSemantics ( Stack  parentMiniNodeStack )
       throws SemanticErrorException
     //////////////////////////////////////////////////////////////////////
     {
       DeclarationSequenceMiniNode  declarationSequenceMiniNode
         = blockMiniNode.getDeclarationSequenceMiniNode ( );

       boolean  containsNested = false;

       if ( declarationSequenceMiniNode != null )
       {
         List  declarationMiniNodeList
           = declarationSequenceMiniNode.getDeclarationMiniNodeList ( );

         Iterator  i = declarationMiniNodeList.iterator ( );

         while ( i.hasNext ( ) )
         {
           if ( i.next ( ) instanceof ProcedureDeclarationMiniNode )
           {
             containsNested = true;
             break;
           }
         }
       }

       this.containsNested = new Boolean ( containsNested );

       parentMiniNodeStack.push ( this );

       if ( blockMiniNode != null )
       {
         blockMiniNode.checkSemantics ( parentMiniNodeStack );
       }

       parentMiniNodeStack.pop ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
