     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.io.LineInputStreamQueue"
     // created 1997-04-26.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.io;

     import com.orbs.open1.mpl.util.Queue;
     import java.io.*;

     /*********************************************************************
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   1997-04-26
     *********************************************************************/

     public class  LineInputStreamQueue extends Queue implements Runnable {
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     private DataInputStream  dataInputStream;
     private Thread           thread;
     private IOException      ioException;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  LineInputStreamQueue ( DataInputStream  dataInputStream ) {
     //////////////////////////////////////////////////////////////////////
       this.dataInputStream = dataInputStream;
     }

     /*********************************************************************
     * Starts a Thread of this.
     *********************************************************************/
     public void  start ( ) {
     //////////////////////////////////////////////////////////////////////
       if ( thread == null ) ( thread = new Thread ( this ) ).start ( );
     }

     /*********************************************************************
     * Calls readLine() on the dataInputStream in an infinite loop,
     * adding incoming lines to the queue.
     * <P>
     * On IOException, breaks out of the loop and makes the queue
     * impatient.  Thus, if an IOException occurs, the remove() call will
     * eventually start returning null.  See popIOException().
     *********************************************************************/
     public void  run ( ) {
     //////////////////////////////////////////////////////////////////////
       try { while ( true ) add ( dataInputStream.readLine ( ) ); }
       catch ( IOException  e ) {
         this.ioException = e;
         setImpatient ( true );
         thread = null;
       }
     }

     /*********************************************************************
     * Stops the Thread of this.<BR>
     * You may wish to call setImpatient(true) after calling this
     * so that the remove() will eventually return null.
     *********************************************************************/
     public void  stop ( ) {
     //////////////////////////////////////////////////////////////////////
       if ( thread != null ) {
         thread.stop ( );
         thread = null;
       }
     }

     /*********************************************************************
     * Returns the last IOException that occurred in the run() loop while
     * reading from the DataInputStream.  Returns null on subsequent
     * calls if no new IOException has occurred since.
     *********************************************************************/
     public IOException  popIOException ( ) {
     //////////////////////////////////////////////////////////////////////
       IOException  ioException = this.ioException;
       this.ioException = null;
       return ioException;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
