     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.io.StreamLib",
     // created 1999-08-15.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.io;

     import java.io.*;
     import java.util.*;

     /*********************************************************************
     *
     * A library of static methods for the manipulation of input and
     * output streams.
     *
     * <P>
     *
     * @version
     *   2000-04-30
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     *********************************************************************/

     public final class  StreamLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private  StreamLib ( ) { }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static void  main ( String [ ]  args )
     //////////////////////////////////////////////////////////////////////
     {
       System.out.println ( System.getProperty ( "file.encoding" ) );
     }

     /*********************************************************************
     * Replaces bytes in a stream as it is being copied from the
     * InputStream to the OutputStream.
     *
     * <P>
     * 
     * Does not automatically close the streams after completion.
     * Maintains an internal buffer with the same size as oldBytes.
     *
     * @return
     *   Returns true if bytes were replaced.
     *********************************************************************/
     public static boolean  replaceBytes (
       InputStream   in,
       OutputStream  out,
       byte [ ]      oldBytes,
       byte [ ]      newBytes )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       byte [ ]  bufBytes = new byte [ oldBytes.length ];

       boolean  hasChanged = false;

       int  i;

       byte  b;

       int  index = 0;

       while ( ( i = in.read ( ) ) > -1 )
       {
         b = ( byte ) i;

         if ( b == oldBytes [ index ] )
         {
           if ( index + 1 < bufBytes.length )
           {
             bufBytes [ index++ ] = b;
           }
           else
           {
             out.write ( newBytes );

             hasChanged = true;

             index = 0;
           }
         }
         else
         {
           out.write ( bufBytes, 0, index );

           out.write ( b );

           index = 0;
         }
       }

       return hasChanged;
     }

     /*********************************************************************
     * Converts the bytes of the InputStream to a String.
     *
     * <P>
     * 
     * Does not automatically close the stream after completion.
     *********************************************************************/
     public static String  toString (
       InputStream  inputStream,
       String       encoding )
       throws IOException, UnsupportedEncodingException
     //////////////////////////////////////////////////////////////////////
     {
       ByteArrayOutputStream  byteArrayOutputStream
         = new ByteArrayOutputStream ( );

       int  i;

       while ( ( i = inputStream.read ( ) ) > -1 )
       {
         byteArrayOutputStream.write ( i );
       }

       return byteArrayOutputStream.toString ( encoding );
     }

     /*********************************************************************
     * Converts the bytes of the InputStream to a String.
     *
     * <P>
     * 
     * Does not automatically close the stream after completion.
     *********************************************************************/
     public static String  toString (
       InputStream  inputStream )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       ByteArrayOutputStream  byteArrayOutputStream
         = new ByteArrayOutputStream ( );

       int  i;

       while ( ( i = inputStream.read ( ) ) > -1 )
       {
         byteArrayOutputStream.write ( i );
       }

       return byteArrayOutputStream.toString ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
