     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.queue.Queue",
     // created 1998-11-23.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.j1d1.util.queue;

     /*********************************************************************
     *
     * An interface for queues.
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1998-11-23
     *********************************************************************/

     public interface  Queue
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     /*********************************************************************
     * Appends the object to the queue and notifies any threads that
     * are blocked in pull().
     *
     * @return
     *   Returns true if the object could be added to the queue.
     *********************************************************************/
     public boolean  append ( Object  o );

     /*********************************************************************
     * Poll this queue to see if an object is available, immediately
     * removing one if so.  If the queue is empty, this method
     * immediately returns null.
     *
     * @return
     *   An object, if one was immediately available, otherwise null.
     *********************************************************************/
     public Object  poll ( );

     /*********************************************************************
     * Remove the next object in this queue, blocking until one becomes
     * available.
     *
     * @throws InterruptedException
     *    If the wait is interrupted.
     *********************************************************************/
     public Object  pull ( ) throws InterruptedException;

     /*********************************************************************
     * Remove the next object in this queue, blocking until one becomes
     * available or the given timeout period expires.
     *
     * @param timeout
     *   If positive, block for up to <I>timeout</I> milliseconds while
     *   waiting for an object to be added to this queue.
     *   If zero, block indefinitely.
     *
     * @return
     *   An object, if one was available within the specified timeout
     *   period, otherwise null.
     *
     * @throws IllegalArgumentException
     *    If the value of the timeout argument is negative.
     * @throws InterruptedException
     *    If the timeout wait is interrupted.
     *********************************************************************/
     public Object  pull ( long  timeout )
       throws IllegalArgumentException, InterruptedException;

     /*********************************************************************
     * Replaces the first occurrence of any equal object in the queue.
     * The new object is placed in the same queue order as the old.
     * If the new object is unique, calls add(o).
     *
     * @return
     *   The old object, if available, otherwise null.
     *********************************************************************/
     public Object  replace ( Object  o );

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
