     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.math.FinanceLib",
     // created 1999-08-15.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.math;

     /*********************************************************************
     *
     * Financial calculations.
     *
     * @version
     *   1999-08-15
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public final class  FinanceLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private  FinanceLib ( ) { }

     /*********************************************************************
     * Test method:  c, r, t; the future value of an annuity followed by
     * the present value calculated three different ways.
     *********************************************************************/
     public static final void  main ( String [ ]  args )
     //////////////////////////////////////////////////////////////////////
     {
       double [ ]  v = new double [ args.length ];
       for ( int  i = 0; i < v.length; i++ )
       {
         v [ i ] = Double.valueOf ( args [ i ] ).doubleValue ( );
       }
       testRetire ( v [ 0 ], v [ 1 ], v [ 2 ], v [ 3 ], v [ 4 ], v [ 5 ] );
     }

     public static final void  testRetire (
       double  desiredSavingsInterestIncome,
       double  savingsInterestRate,
       double  inflationRate,
       double  taxRate,
       double  investmentInterestRate,
       double  yearsOfSaving )
     //////////////////////////////////////////////////////////////////////
     {
     
//     desiredSavingsInterestIncome
//       = savings
//       * ( savingsInterestRate * ( 1.0 - taxRate ) - inflationRate )
//       / ( 1.0 + inflationRate );
     
       double  savings
         = desiredSavingsInterestIncome * ( 1.0 + inflationRate )
         / ( savingsInterestRate * ( 1.0 - taxRate ) - inflationRate );

       double  futureValueSavings
         = savings * Math.pow ( 1.0 + inflationRate, yearsOfSaving );

       double  annualSavings = annualSavingsNeeded (
         futureValueSavings, investmentInterestRate, yearsOfSaving );

       System.out.println ( savings );
       System.out.println ( futureValueSavings );
       System.out.println ( annualSavings );
       System.out.println ( futureValueAnnuity (
         annualSavings, investmentInterestRate, yearsOfSaving ) );
       System.out.println ( presentValueAnnuity (
         annualSavings, investmentInterestRate, yearsOfSaving ) );
     }

     /*********************************************************************
     * Test method.  Prints the future value of an annuity followed by
     * the present value calculated three different ways.
     *
     * @param c Annual cash income starting one year from today.
     * @param r Annual interest earned on income.
     * @param t Number of years of cash income.
     *********************************************************************/
     public static final void  testAnnuity (
       double  C,
       double  r,
       double  T )
     //////////////////////////////////////////////////////////////////////
     {
       double  FV = futureValueAnnuity ( C, r, T );
       System.out.println ( FV );
       System.out.println ( presentValue ( FV, r, T ) );
       System.out.println ( presentValueAnnuity ( C, r, T ) );
       double [ ]  c = new double [ ( int ) T ];
       for ( int  i = 0; i < T; i++ ) c [ i ] = C;
       System.out.println ( presentValue ( c, r ) );
     }

     /*********************************************************************
     * The future value of a cash flow received today.
     *
     * @param c Cash flow today.
     * @param r Inflation rate.
     * @param t Number of years from today when the value is evaluated.
     *********************************************************************/
     public static final double  futureValue (
       double  c,
       double  r,
       double  t )
     //////////////////////////////////////////////////////////////////////
     {
       return c * Math.pow ( 1.0 + r, t );
     }

     /*********************************************************************
     * Calculates the future value of an annuity.
     *
     * @param c Annual cash income starting one year from today.
     * @param r Annual interest earned on income.
     * @param t Number of years of cash income.
     *********************************************************************/
     public static final double  futureValueAnnuity (
       double  c,
       double  r,
       double  t )
     //////////////////////////////////////////////////////////////////////
     {
       return c * ( ( Math.pow ( 1.0 + r, t ) - 1.0 ) / r );
     }

     /*********************************************************************
     * The discounted value of a single cash flow received in the future.
     *
     * @param c Cash flow received in the future
     * @param r Inflation or annual interest.
     * @param t Number of years from today when the cash flow is received.
     *********************************************************************/
     public static final double  presentValue (
       double  c,
       double  r,
       double  t )
     //////////////////////////////////////////////////////////////////////
     {
       return c / Math.pow ( 1.0 + r, t );
     }

     /*********************************************************************
     * The discounted value of varying annual cash flows.
     *
     * @param c Array of annual cash income starting one year from today.
     * @param r Annual interest earned on income.
     *********************************************************************/
     public static final double  presentValue (
       double [ ]  c,
       double      r )
     //////////////////////////////////////////////////////////////////////
     {
       double  sum = 0.0;
       for ( int  i = 0; i < c.length; i++ )
       {
         sum += presentValue ( c [ i ], r, i + 1 );
       }
       return sum;
     }

     /*********************************************************************
     * Calculates the present value of an annuity.
     *
     * @param c Annual cash income starting one year from today.
     * @param r Inflation or annual interest.
     * @param t Number of years of cash income.
     *********************************************************************/
     public static final double  presentValueAnnuity (
       double  c,
       double  r,
       double  t )
     //////////////////////////////////////////////////////////////////////
     {
       return c * ( 1.0 - 1.0 / Math.pow ( 1.0 + r, t ) ) / r;
     }

     /*********************************************************************
     * Calculates the annual savings necessary to accumulate a specified
     * value in the future.
     *
     * @param f Future value desired.
     * @param r Annual interest.
     * @param t Number of years of savings.
     *********************************************************************/
     public static final double  annualSavingsNeeded (
       double  f,
       double  r,
       double  t )
     //////////////////////////////////////////////////////////////////////
     {
       return f * r / ( Math.pow ( 1.0 + r, t ) - 1.0 );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }

