     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.net.Downloader",
     // created 1998-11-30.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.net;

     import java.io.*;
     import java.net.*;

     import com.orbs.open1.mpl.lang.StringLib;

     /*********************************************************************
     *
     * Downloads the content of a URL to a local File.
     * 
     * @version
     *   1998-11-30
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  Downloader
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     /*********************************************************************
     * Downloads the content at a URL to a local destination File.
     *********************************************************************/
     public static void  download ( URL  url, File  dest )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       BufferedInputStream   in  = null;
       BufferedOutputStream  out = null;

       try
       {
         in  = new BufferedInputStream ( url.openStream ( ) );
         out = new BufferedOutputStream (
           new FileOutputStream ( dest ) );

         int  i;
         while ( ( i = in.read ( ) ) > -1 ) out.write ( ( byte ) i );
       }
       finally
       {
         try { out.close ( ); } catch ( Exception  ex1 ) { }
         try { in.close  ( ); } catch ( Exception  ex1 ) { }
       }
     }

     public static boolean  downloadResourceToDir (
       URL  codebaseURL, String  name, File  destDir )
     //////////////////////////////////////////////////////////////////////
     {
       if ( ( codebaseURL == null )
         || ( name        == null )
         || ( destDir     == null ) )
//       || !destDir.exists ( )
//       || !destDir.isDirectory ( ) )
         return false;

       InputStream           inputStream = null;
       BufferedInputStream   in          = null;
       BufferedOutputStream  out         = null;

       try
       {
         // We create the file twice to make sure the path is resolved.
//System.out.println ( "destDir:  " + destDir.getCanonicalPath ( ) );
//System.out.println ( "name:  " + name );
         File  cacheFile = new File ( destDir, name );
//System.out.println ( "cacheFile1:  " + cacheFile.getCanonicalPath ( ) );
         cacheFile = new File ( cacheFile.getCanonicalPath ( ) );
//System.out.println ( "cacheFile2:  " + cacheFile.getCanonicalPath ( ) );

         File  parentFile = new File ( cacheFile.getParent ( ) );
//System.out.println ( "parentFile:  " + parentFile.getCanonicalPath ( ) );
//System.out.println ( "mkdirs:  " + parentFile.mkdirs ( ) );
         parentFile.mkdirs ( );

         inputStream = downloadResource ( codebaseURL, name );
         if ( inputStream == null )
         {
           System.out.println ( "  Download of \"" + name + "\" from\n"
             + "    \"" + codebaseURL + "\" failed." );
           return false;
         }

         in  = new BufferedInputStream ( inputStream );
         out = new BufferedOutputStream (
           new FileOutputStream ( cacheFile ) );

         int  i;
         while ( ( i = in.read ( ) ) > -1 ) out.write ( ( byte ) i );

System.out.println ( "  Successfully downloaded and saved." );

         return true;
       }
       catch ( Exception  ex )
       {
         ex.printStackTrace ( );
       }
       finally
       {
         try { out.close          ( ); } catch ( Exception  ex1 ) { }
         try { in.close           ( ); } catch ( Exception  ex1 ) { }
         try { inputStream.close  ( ); } catch ( Exception  ex1 ) { }
         return false;
       }
     }

     /*********************************************************************
     * Returns null if the codebaseURL is null.
     * Returns null upon failure.
     *********************************************************************/
     public static InputStream  downloadResource (
       URL  codebaseURL, String  name )
     //////////////////////////////////////////////////////////////////////
     {
       if ( codebaseURL == null ) return null;

       InputStream  inputStream = null;
       
       try
       {
         String  urlName
           = replaceSpaces ( replaceSeparators ( name ) );

         URL  url = new URL ( codebaseURL, urlName );
System.out.println ( "Downloading \"" + name + "\" from\n  \"" + url + "\"" );

         URLConnection  urlConnection = url.openConnection ( );

         inputStream = urlConnection.getInputStream ( );

         return inputStream;
       }
       catch ( IOException  ex )
       {
         try { inputStream.close ( ); } catch ( Exception  ex1 ) { }
         return null;
       }
     }

     /*********************************************************************
     * Replaces every instance of File.separator with a forward slash
     * character.  This is used for converting a local path name to
     * a URL path name.
     *********************************************************************/
     public static String  replaceSeparators ( String  localPath )
     //////////////////////////////////////////////////////////////////////
     {
       return StringLib.replace ( localPath, File.separator, "/" );
     }

     /*********************************************************************
     * Replaces every instance of space (" ") with the
     * x-www-form-urlencoded equivalent ("%20").
     * This is used for converting a remote filename with spaces so that
     * it can be downloaded via a HTTP request.
     *********************************************************************/
     public static String  replaceSpaces ( String  remoteFilename )
     //////////////////////////////////////////////////////////////////////
     {
       return StringLib.replace ( remoteFilename, " ", "%20" );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
