     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.net.news.NewsFrame",
     // created 1997-05-09.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.net.news;

     import java.awt.*;
     import java.io.IOException;
     import java.util.*;

     /*********************************************************************
     * <P>
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   1997-05-09
     *********************************************************************/

     public class  NewsFrame extends Frame {
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static final String  COPYRIGHT
       = "NewsFrame \u00A9 1997 David Wallace Croft";
     public static final String  INFO
       = COPYRIGHT + '\n'
       + "croft@alumni.caltech.edu\n"
       + "http://www.alumni.caltech.edu/~croft/\n";
     public static final String  DEFAULT_HOST = "news.hughes.net";
     public static final String  DEFAULT_GROUP
       = "alt.binaries.pictures.anime";

     private NNTPSocket  nntpSocket;
     private TextField   server_TextField = new TextField ( DEFAULT_HOST );
     private TextField   group_TextField = new TextField ( DEFAULT_GROUP );
     private Choice      newsgroup_Choice;
     private TextArea    log_TextArea = new TextArea ( INFO );
     private int         newsgroup_index;
     private String      current_group;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static void  main ( String [ ]  args ) {
     //////////////////////////////////////////////////////////////////////
       new NewsFrame ( );
     }

     public  NewsFrame ( ) {
     //////////////////////////////////////////////////////////////////////
       newsgroup_Choice_load ( 0 );
       setTitle ( COPYRIGHT );
       add ( server_TextField );
       add ( group_TextField );
       add ( newsgroup_Choice );
       add ( log_TextArea );
       reshape ( 0, 0, 640, 480 );
       show ( );
     }

     public synchronized void  newsgroup_Choice_load (
       int  newsgroup_index ) {
     //////////////////////////////////////////////////////////////////////
System.out.println ( "newsgroup_Choice(" + newsgroup_index + ")" );
       Choice  old_Choice = newsgroup_Choice;
       add ( newsgroup_Choice = new Choice ( ) );
       if ( newsgroup_index < 0 ) newsgroup_index = 0;
       this.newsgroup_index = newsgroup_index;
       newsgroup_Choice.addItem ( "Update newsgroup list" );
       newsgroup_Choice.addItem ( "Previous 20" );
       newsgroup_Choice.addItem ( "Next 20" );
       newsgroup_Choice.addItem ( "---" );
       if ( nntpSocket == null ) return;
       Vector  newsgroup_Vector = nntpSocket.newsgroup_Vector ( );
       if ( newsgroup_Vector == null ) return;
       int  newsgroup_Vector_size = newsgroup_Vector.size ( );
       if ( newsgroup_Vector_size - 1 < newsgroup_index ) return;
       int  max_index = newsgroup_index + 20;
       if ( max_index > newsgroup_Vector_size - 1 ) {
         max_index = newsgroup_Vector_size - 1;
       }
       for ( int  i = newsgroup_index; i < max_index; i++ ) {
         Newsgroup  newsgroup
           = ( Newsgroup ) newsgroup_Vector.elementAt ( i );
         newsgroup_Choice.addItem ( newsgroup.group_name ( ) );
       }
       synchronized ( old_Choice ) {
         if ( old_Choice != null ) remove ( old_Choice );
         layout ( );
       }
     }

     public boolean  handleEvent ( Event  event ) {
     //////////////////////////////////////////////////////////////////////
       if ( event.id == Event.WINDOW_DESTROY ) {
         try { if ( nntpSocket != null ) nntpSocket.close ( ); }
         catch ( IOException  e ) { log ( e.getMessage ( ) ); }
         nntpSocket = null;
         dispose ( );
         return true;
       }
       if ( event.id == Event.ACTION_EVENT ) {
         if ( event.target == server_TextField ) {
           try {
             if ( nntpSocket != null ) nntpSocket.command_QUIT ( );
             nntpSocket = new NNTPSocket ( server_TextField.getText ( ) );
             log ( nntpSocket.response_code ( ) );
           } catch ( IOException  e ) { log ( e.getMessage ( ) ); }
           return true;
         }
         if ( event.target == group_TextField ) {
           if ( nntpSocket == null ) {
             log ( "You are not currently connected." );
             return true;
           }
           try {
             String  response;
             log ( response = nntpSocket.command_GROUP (
               group_TextField.getText ( ) ) );
           } catch ( IOException  e ) { log ( e.getMessage ( ) ); }
           return true;
         }
         if ( event.target == newsgroup_Choice ) {
           if ( nntpSocket == null ) {
             log ( "You are not currently connected." );
           }
           int  index = newsgroup_Choice.getSelectedIndex ( );
           if ( index == 0 ) {
             synchronized ( newsgroup_Choice ) {
               String  response;
               try {
                 log ( response = nntpSocket.command_LIST ( ) );
               } catch ( IOException  e ) {
                 log ( e.getMessage ( ) );
                 return true;
               }
               if ( !response.startsWith ( "21" ) ) return true;
               newsgroup_Choice_load ( 0 );
             }
           } else if ( index == 1 ) {
             newsgroup_Choice_load ( newsgroup_index - 20 );
           } else if ( index == 2 ) {
             newsgroup_Choice_load ( newsgroup_index + 20 );
           } else if ( index > 3 ) {
             String  response;
             try {
               log ( response = nntpSocket.command_GROUP (
                 newsgroup_Choice.getSelectedItem ( ) ) );
             } catch ( IOException  e ) {
               log ( e.getMessage ( ) );
               return true;
             }
           }
           return true;
         }
       }
       return super.handleEvent ( event );
     }

     public void  layout ( ) {
     //////////////////////////////////////////////////////////////////////
       Insets  insets = insets ( );
       Rectangle  bounds = bounds ( );
       int  usable_width  = bounds.width  - insets.left - insets.right;
       int  usable_height = bounds.height - insets.top - insets.bottom;
       int  server_TextField_height
         = server_TextField.preferredSize ( ).height;
       int  group_TextField_height
         = group_TextField.preferredSize ( ).height;
       int  newsgroup_Choice_height
         = newsgroup_Choice.preferredSize ( ).height;
       server_TextField.reshape (
         insets.left, insets.top, usable_width, server_TextField_height );
       group_TextField.reshape ( insets.left,
         insets.top + group_TextField_height, usable_width,
         group_TextField_height );
       newsgroup_Choice.reshape ( insets.left,
         insets.top + server_TextField_height + group_TextField_height,
         usable_width, newsgroup_Choice_height );
       log_TextArea.reshape ( insets.left,
         insets.top + server_TextField_height + group_TextField_height
         + newsgroup_Choice_height,
         usable_width,
         usable_height - server_TextField_height
         - group_TextField_height - newsgroup_Choice_height );
     }

     public void  log ( String  line ) {
     //////////////////////////////////////////////////////////////////////
       log_TextArea.append ( line + '\n' );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
