     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.HashtableLib",
     // created 1998-10-04.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util;

     import java.util.*;

     /*********************************************************************
     *
     * Static method library to manipulate Hashtable objects.
     *
     * @version
     *   1998-10-04
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  HashtableLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private  HashtableLib ( ) { }

     /*********************************************************************
     * Creates a Hashtable of just the new and updated elements.
     * <P>
     * Returns a Hashtable containing those elements in the new
     * Hashtable that were not in the old Hashtable plus those elements
     * in new Hashtable that were in the old Hashtable but with different
     * values.
     * <P>
     * Assumes that the Hashtable value objects have overridden
     * their equals() method for comparison.
     *
     * @return
     *   Key and value objects returned are shared from the newHashtable.
     *   Returns null if newHashtable is null.
     *   Returns a shallow clone of newHashtable if oldHashtable is null.
     *********************************************************************/
     public static Hashtable  hashtableDelta (
       Hashtable  oldHashtable, Hashtable  newHashtable )
     //////////////////////////////////////////////////////////////////////
     {
       if ( newHashtable == null ) return null;
       if ( oldHashtable == null )
       {
         return ( Hashtable ) newHashtable.clone ( );
       }

       Hashtable  deltaHashtable = new Hashtable ( );

       Enumeration  e = newHashtable.keys ( );
       while ( e.hasMoreElements ( ) )
       {
         Object  key = e.nextElement ( );
         Object  value = oldHashtable.get ( key );
         if ( ( value == null )
           || !value.equals ( newHashtable.get ( key ) ) )
         {
           deltaHashtable.put ( key, newHashtable.get ( key ) );
         }
       }

       return deltaHashtable;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
