     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.Queue",
     // created 1997-04-26.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util;

     import java.util.Vector;

     /*********************************************************************
     * <P>
     * The default value for impatient is false.<BR>
     * The default value for rejecting is false.
     * <P>
     * @version
     *   1997-04-26
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     *********************************************************************/

     public class  Queue {
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     private Vector   queue     = new Vector ( );
     private boolean  impatient = false;
     private boolean  rejecting = false;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * A rejecting Queue will not add the item.
     * Returns false if the item was not added.
     *********************************************************************/
     public boolean  add ( Object  item ) {
     //////////////////////////////////////////////////////////////////////
       if ( rejecting ) return false;
       synchronized ( queue ) {
         queue.addElement ( item );
         queue.notify ( );
       }
       return true;
     }

     /*********************************************************************
     * An impatient Queue returns null when it is empty.<BR>
     * A patient Queue will wait.
     *********************************************************************/
     public Object  remove ( ) {
     //////////////////////////////////////////////////////////////////////
       synchronized ( queue ) {
         while ( !impatient && queue.isEmpty ( ) ) {
           try { queue.wait ( ); } catch ( InterruptedException  e ) { }
         }
         if ( queue.isEmpty ( ) ) return null;
         Object  item = queue.firstElement ( );
         queue.removeElement ( item );
         return item;
       }
     }

     public boolean  isImpatient ( ) {
     //////////////////////////////////////////////////////////////////////
       return impatient;
     }

     /*********************************************************************
     * Notifies any waiting remove() calls in the process.
     *********************************************************************/
     public void  setImpatient ( boolean  impatient ) {
     //////////////////////////////////////////////////////////////////////
       this.impatient = impatient;
       synchronized ( queue ) { queue.notify ( ); }
     }

     public boolean  isRejecting ( ) {
     //////////////////////////////////////////////////////////////////////
       return rejecting;
     }

     public void  setRejecting ( boolean  rejecting ) {
     //////////////////////////////////////////////////////////////////////
       this.rejecting = rejecting;
     }

     public boolean  isEmpty ( ) {
     //////////////////////////////////////////////////////////////////////
       return queue.isEmpty ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
