     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.util.cache.MemoryMapCache",
     // created 1999-04-11.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util.cache;

     import java.io.*;
     import java.util.*;

     import com.orbs.open1.mpl.j1d1.util.id.*;

     /*********************************************************************
     * A Cache implementation that is backed by a memory Map.
     *
     * <P>
     *
     * The InputStreams are read and stored to the Map as byte arrays.
     *
     * @see
     *   Cache
     * @see
     *   java.util.WeakHashMap
     * @see
     *   com.orbs.open.a.mpl.util.SoftHashMap
     *
     * @version
     *   1999-04-24
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  MemoryMapCache implements Cache
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private Map  map;

     //////////////////////////////////////////////////////////////////////
     // Constructor method
     //////////////////////////////////////////////////////////////////////

     public  MemoryMapCache ( Map  map )
     //////////////////////////////////////////////////////////////////////
     {
       if ( map == null )
       {
         throw new IllegalArgumentException ( "null map" );
       }

       this.map = map;
     }

     //////////////////////////////////////////////////////////////////////
     // Cache interface methods
     //////////////////////////////////////////////////////////////////////

     public ID  validate ( ID  id, ContentAccessor  contentAccessor )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       if ( isAvailable ( id ) ) return id;

       InputStream  inputStream = contentAccessor.getInputStream ( );

       if ( inputStream == null ) return null;

       return store ( inputStream );
     }

     public ID  store ( InputStream  in ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       ID  id = new IntID ( );

       map.put ( id, CacheLib.toByteArray ( in ) );

       return id;
     }

     public InputStream  retrieve ( ID  id ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       byte [ ]  content = ( byte [ ] ) map.get ( id );

       if ( content == null ) return null;

       return new ByteArrayInputStream ( content );
     }

     public boolean  isAvailable ( ID  id )
     //////////////////////////////////////////////////////////////////////
     {
       return map.containsKey ( id );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
