     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.util.cache.secure.SecureInputStream",
     // created 1999-04-13.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 2000 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util.cache.secure;

     import java.io.*;
     import java.security.*;

     /*********************************************************************
     * A FilterInputStream which will throw a SecurityException if the
     * stream content does not have the digest expected.
     *
     * @see
     *   java.security.DigestInputStream
     * @see
     *   java.security.MessageDigest
     *
     * @version
     *   1999-04-13
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  SecureInputStream extends FilterInputStream
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     /** The expected digest. */
     private byte [ ]  digest;

     //////////////////////////////////////////////////////////////////////
     // Constructor method
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * @param  algorithm
     *   The MessageDigest algorithm to use.
     * @param  digest
     *   The expected digest.
     *********************************************************************/
     public  SecureInputStream (
       InputStream  inputStream,
       String       algorithm,
       byte [ ]     digest )
       throws NoSuchAlgorithmException
     //////////////////////////////////////////////////////////////////////
     {
       super ( new DigestInputStream ( inputStream,
         MessageDigest.getInstance ( algorithm ) ) );

       this.digest = digest;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * @throws  SecurityException
     *   If the end of the stream is reached and the digest is not what
     *   is expected.
     *********************************************************************/
     public int  read ( ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       int  i = super.read ( );

       if ( i < 0 ) checkDigest ( );

       return i;
     }

     /*********************************************************************
     * @throws  SecurityException
     *   If the end of the stream is reached and the digest is not what
     *   is expected.
     *********************************************************************/
     public int  read ( byte [ ]  byteArray, int  offset, int  length )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       int  i = super.read ( byteArray, offset, length );

       if ( i < length ) checkDigest ( );

       return i;
     }

     private void  checkDigest ( ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       if ( !MessageDigest.isEqual ( digest,
           ( ( DigestInputStream ) in ).getMessageDigest ( ).digest ( ) ) )
       {
         throw new IOException ( "bad message digest" );
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
