     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.state.State",
     // created 1999-02-06.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util.state;

     /*********************************************************************
     * The state of an object may be communicated by a State object.
     * Each State object has a unique key, usually the object or its unique
     * identifier whose state or a portion of its state is reflected by
     * this State object.
     *
     * <P>
     *
     * State objects are considered equal if their classes and keys are
     * equal.  This makes a State object useful in Set collections where
     * the state or the latest subset of the state of the key object should
     * should only be contained once.  One application is the queued
     * transmission of object state information wherein only the latest
     * state data should be retained.
     *
     * @see
     *   StateLib
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1999-02-06
     *********************************************************************/

     public interface State
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     /*********************************************************************
     * Returns the State key, usually the object or its unique identifier
     * whose state or a portion of its state is reflected by this State
     * object.
     *********************************************************************/
     public Object  getKey ( );

     /*********************************************************************
     * Returns true if the classes and State keys are equal.
     *********************************************************************/
     public boolean  equals ( Object  other );

     /*********************************************************************
     * Returns the hash code of the State key.
     *********************************************************************/
     public int  hashCode ( );

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
